//          File: graph.cpp
//        Author: Fred Sisenda
// Last Modified: February 29, 2012

#include "vertex.cpp"
#include <vector>

using namespace std;

class Graph{
	
	vector<Vertex*> vertices;
	
public:
	// Implicit constructor
	// Graph()
	
	// Destructor: deletes vertices in graph 
	~Graph(){
		vector<Vertex*>::const_iterator lIter;
		for(lIter=vertices.begin(); lIter!=vertices.end(); lIter++)
			delete *lIter;
	}


	// returns the pointer to vertex with specified id
	// returns NULL if no vertex with id exists in graph
	Vertex* getVertexById(int id){
		vector<Vertex*>::const_iterator lIter;
		for(lIter=vertices.begin(); lIter!=vertices.end(); lIter++)
			if ((*lIter)->getId()==id) return *lIter;
		// vertex with specified id not found
		return NULL;
	}


	// returns the pointer to nth vertex in graph 
	// returns NULL if n exceeds range
	Vertex* getVertexByIndex(unsigned int n){
		if (n>=vertices.size()) return NULL;
		return vertices[n];
	}


	// return current number of vertices in graph
	int getNumVertices(){
		return vertices.size();
	}


	// add vertex with specified id to graph
	// if vertex with id already exists, does nothing, returns false'
	// otherwise the vertex is added, returns 'true'
	bool addVertex(int id){ 
		// check if the vertex is already in the graph
		if ( getVertexById(id) != NULL ) return false;
	
		// allocate memory for new vertex
		Vertex* newVertex = new Vertex(id);
	
		// add vertex to vertex list
		vertices.push_back(newVertex);
		return true;
	}


	// adds edge from source to dest of weight w if source and destination are in graph
	// returns true if successful'
	// returns false if source or destination does not exist in graph
	bool addEdge(int sourceId, int destId, int w) {
		// find source vertex
		Vertex* source = getVertexById(sourceId);
		if (source == NULL) return false;
	
		// find destination vertex
		Vertex* dest = getVertexById(destId);
		if (dest == NULL) return false;
	
		// add edge
		source->addEdgeTo(dest,w);
		return true;
	}


	// print adjacency of all vertices in the graph
	void print(){
		vector<Vertex*>::const_iterator lIter;
		for(lIter=vertices.begin(); lIter!=vertices.end(); lIter++)
			(*lIter)->printEdges();
	}
};